@@ -43,6 +43,12 @@ namespace infomap {
 
 using std::make_pair;
 
+void Network::init()
+{
+  initValidHeadings();
+  m_multilayerStateIdBitShift = static_cast<unsigned int>(std::ceil(std::log2(m_config.matchableMultilayerIds)));
+}
+
 void Network::initValidHeadings()
 {
   auto& headingsPajek = m_validHeadings["pajek"];
@@ -373,6 +379,11 @@ std::string Network::parseMultilayerLinks(std::ifstream& file)
 {
   Log() << "  Parsing multilayer links...\n"
         << std::flush;
+
+  if (m_config.matchableMultilayerIds > 0) {
+    Log() << "  Creating matchable state ids using: nodeId << (log2(" << m_config.matchableMultilayerIds << ") + 1) | layerId\n";
+  }
+
   std::string line;
   while (!std::getline(file, line).fail()) {
     if (line.length() == 0 || line[0] == '#')
@@ -398,6 +409,11 @@ std::string Network::parseMultilayerIntraLinks(std::ifstream& file)
 {
   Log() << "  Parsing intra-layer links...\n"
         << std::flush;
+
+  if (m_config.matchableMultilayerIds > 0) {
+    Log() << "  Creating matchable state ids using: nodeId << (log2(" << m_config.matchableMultilayerIds << ") + 1) | layerId\n";
+  }
+
   std::string line;
   while (!std::getline(file, line).fail()) {
     if (line.length() == 0 || line[0] == '#')
@@ -1071,7 +1087,16 @@ unsigned int Network::addMultilayerNode(unsigned int layerId, unsigned int physi
   if (it != layerIt.end()) {
     return it->second;
   }
-  auto ret = addStateNodeWithAutogeneratedId(physicalId);
+
+  bool matchableMultilayerIds = m_config.matchableMultilayerIds != 0;
+
+  if (matchableMultilayerIds && layerId > m_config.matchableMultilayerIds) {
+    throw InputDomainError(io::Str() << "Cannot add node with layer " << layerId << " to network with matchable multilayer ids using largest layer id " << m_config.matchableMultilayerIds);
+  }
+
+  auto ret = matchableMultilayerIds
+      ? addStateNodeWithDeterministicId(physicalId, layerId, m_multilayerStateIdBitShift)
+      : addStateNodeWithAutogeneratedId(physicalId);
   auto& stateNode = ret.first->second;
   stateNode.layerId = layerId;
   stateNode.weight = weight;
